@@ -50,6 +50,10 @@
 
 #include "kafkacat.h"
 
+#if RD_KAFKA_VERSION >= 0x01040000
+#define ENABLE_TXNS 1
+#endif
+
 
 struct conf conf = {
         .run = 1,
@@ -59,6 +63,8 @@ struct conf conf = {
         .msg_size = 1024*1024,
         .null_str = "NULL",
         .fixed_key = NULL,
+        .metadata_timeout = 5,
+        .offset = RD_KAFKA_OFFSET_INVALID,
 };
 
 static struct stats {
@@ -84,7 +90,7 @@ int part_stop_thres = 0;
  * Fatal error: print error and exit
  */
 void RD_NORETURN fatal0 (const char *func, int line,
-						 const char *fmt, ...) {
+                         const char *fmt, ...) {
         va_list ap;
         char buf[1024];
 
@@ -127,20 +133,27 @@ void error0 (int exitonerror, const char *func, int line, const char *fmt, ...)
 static void dr_msg_cb (rd_kafka_t *rk, const rd_kafka_message_t *rkmessage,
                        void *opaque) {
         static int say_once = 1;
+        int32_t broker_id = -1;
 
         if (rkmessage->err) {
                 KC_INFO(1, "Delivery failed for message: %s\n",
-                     rd_kafka_err2str(rkmessage->err));
+                        rd_kafka_err2str(rkmessage->err));
                 stats.tx_err_dr++;
                 return;
         }
 
-        KC_INFO(3, "Message delivered to partition %"PRId32" (offset %"PRId64")\n",
-             rkmessage->partition, rkmessage->offset);
+#if RD_KAFKA_VERSION >= 0x010500ff
+        broker_id = rd_kafka_message_broker_id(rkmessage);
+#endif
+
+        KC_INFO(3,
+                "Message delivered to partition %"PRId32" (offset %"PRId64") "
+                "on broker %"PRId32"\n",
+                rkmessage->partition, rkmessage->offset, broker_id);
 
         if (rkmessage->offset == 0 && say_once) {
                 KC_INFO(3, "Enable message offset reporting "
-                     "with '-X topic.produce.offset.report=true'\n");
+                        "with '-X topic.produce.offset.report=true'\n");
                 say_once = 0;
         }
         stats.tx_delivered++;
@@ -165,7 +178,7 @@ static void produce (void *buf, size_t len,
 
                 if (!conf.run)
                         KC_FATAL("Program terminated while "
-                              "producing message of %zd bytes", len);
+                                 "producing message of %zd bytes", len);
 
                 err = rd_kafka_producev(
                         conf.rk,
@@ -184,7 +197,7 @@ static void produce (void *buf, size_t len,
 
                 if (err != RD_KAFKA_RESP_ERR__QUEUE_FULL)
                         KC_FATAL("Failed to produce message (%zd bytes): %s",
-                              len, rd_kafka_err2str(err));
+                                 len, rd_kafka_err2str(err));
 
                 stats.tx_err_q++;
 
@@ -207,8 +220,8 @@ static ssize_t produce_file (const char *path) {
         int fd;
         void *ptr;
         struct stat st;
-		ssize_t sz;
-		int msgflags = 0;
+        ssize_t sz;
+        int msgflags = 0;
 
         if ((fd = _COMPAT(open)(path, O_RDONLY)) == -1) {
                 KC_INFO(1, "Failed to open %s: %s\n", path, strerror(errno));
@@ -234,43 +247,43 @@ static ssize_t produce_file (const char *path) {
                 _COMPAT(close)(fd);
                 return -1;
         }
-		sz = st.st_size;
-		msgflags = RD_KAFKA_MSG_F_COPY;
+        sz = st.st_size;
+        msgflags = RD_KAFKA_MSG_F_COPY;
 #else
-		ptr = malloc(st.st_size);
-		if (!ptr) {
-			KC_INFO(1, "Failed to allocate message for %s: %s\n",
-				 path, strerror(errno));
-			_COMPAT(close)(fd);
-			return -1;
-		}
+        ptr = malloc(st.st_size);
+        if (!ptr) {
+                KC_INFO(1, "Failed to allocate message for %s: %s\n",
+                        path, strerror(errno));
+                _COMPAT(close)(fd);
+                return -1;
+        }
 
-		sz = _read(fd, ptr, st.st_size);
-		if (sz < st.st_size) {
-			KC_INFO(1, "Read failed for %s (%zd/%zd): %s\n",
-				 path, sz, (size_t)st.st_size, sz == -1 ? strerror(errno) :
-				 "incomplete read");
-			free(ptr);
-			close(fd);
-			return -1;
-		}
-		msgflags = RD_KAFKA_MSG_F_FREE;
+        sz = _read(fd, ptr, st.st_size);
+        if (sz < st.st_size) {
+                KC_INFO(1, "Read failed for %s (%zd/%zd): %s\n",
+                        path, sz, (size_t)st.st_size, sz == -1 ? strerror(errno) :
+                        "incomplete read");
+                free(ptr);
+                close(fd);
+                return -1;
+        }
+        msgflags = RD_KAFKA_MSG_F_FREE;
 #endif
 
         KC_INFO(4, "Producing file %s (%"PRIdMAX" bytes)\n",
-             path, (intmax_t)st.st_size);
-		produce(ptr, sz, NULL, 0, msgflags);
+                path, (intmax_t)st.st_size);
+        produce(ptr, sz, conf.fixed_key, conf.fixed_key_len, msgflags);
 
-		_COMPAT(close)(fd);
+        _COMPAT(close)(fd);
 
-		if (!(msgflags & RD_KAFKA_MSG_F_FREE)) {
+        if (!(msgflags & RD_KAFKA_MSG_F_FREE)) {
 #ifndef _MSC_VER
-			munmap(ptr, st.st_size);
+                munmap(ptr, st.st_size);
 #else
-			free(ptr);
+                free(ptr);
 #endif
-		}
-		return sz;
+        }
+        return sz;
 }
 
 
@@ -283,6 +296,14 @@ static void producer_run (FILE *fp, char **paths, int pathcnt) {
         size_t  size = 0;
         ssize_t len;
         char    errstr[512];
+        char    tmp[2];
+
+        size = sizeof(tmp);
+        if (rd_kafka_conf_get(conf.rk_conf, "transactional.id",
+                              tmp, &size) == RD_KAFKA_CONF_OK && size > 1) {
+                KC_INFO(1, "Using transactional producer\n");
+                conf.txn = 1;
+        }
 
         /* Assign per-message delivery report callback. */
         rd_kafka_conf_set_dr_msg_cb(conf.rk_conf, dr_msg_cb);
@@ -295,6 +316,23 @@ static void producer_run (FILE *fp, char **paths, int pathcnt) {
         if (!conf.debug && conf.verbosity == 0)
                 rd_kafka_set_log_level(conf.rk, 0);
 
+#if ENABLE_TXNS
+        if (conf.txn) {
+                rd_kafka_error_t *error;
+
+                error = rd_kafka_init_transactions(conf.rk,
+                                                   conf.metadata_timeout*1000);
+                if (error)
+                        KC_FATAL("init_transactions(): %s",
+                                 rd_kafka_error_string(error));
+
+                error = rd_kafka_begin_transaction(conf.rk);
+                if (error)
+                        KC_FATAL("begin_transaction(): %s",
+                                 rd_kafka_error_string(error));
+        }
+#endif
+
         /* Create topic */
         if (!(conf.rkt = rd_kafka_topic_new(conf.rk, conf.topic,
                                             conf.rkt_conf)))
@@ -318,7 +356,7 @@ static void producer_run (FILE *fp, char **paths, int pathcnt) {
                         conf.exitcode = 1;
                 else if (good < pathcnt)
                         KC_INFO(1, "Failed to produce from %i/%i files\n",
-                             pathcnt - good, pathcnt);
+                                pathcnt - good, pathcnt);
 
         } else {
                 /* Read messages from input, delimited by conf.delim */
@@ -393,7 +431,7 @@ static void producer_run (FILE *fp, char **paths, int pathcnt) {
                         if (conf.flags & CONF_F_TEE &&
                             fwrite(sbuf, orig_len, 1, stdout) != 1)
                                 KC_FATAL("Tee write error for message of %zd bytes: %s",
-                                      orig_len, strerror(errno));
+                                         orig_len, strerror(errno));
 
                         if (msgflags & RD_KAFKA_MSG_F_FREE) {
                                 /* rdkafka owns the allocated buffer
@@ -410,10 +448,38 @@ static void producer_run (FILE *fp, char **paths, int pathcnt) {
                 if (conf.run) {
                         if (!feof(fp))
                                 KC_FATAL("Unable to read message: %s",
-                                      strerror(errno));
+                                         strerror(errno));
                 }
         }
 
+#if ENABLE_TXNS
+        if (conf.txn) {
+                rd_kafka_error_t *error;
+                const char *what;
+
+                if (conf.term_sig) {
+                        KC_INFO(0,
+                                "Aborting transaction due to "
+                                "termination signal\n");
+                        what = "abort_transaction()";
+                        error = rd_kafka_abort_transaction(
+                                conf.rk, conf.metadata_timeout * 1000);
+                } else {
+                        KC_INFO(1, "Committing transaction\n");
+                        what = "commit_transaction()";
+                        error = rd_kafka_commit_transaction(
+                                conf.rk, conf.metadata_timeout * 1000);
+                        if (!error)
+                                KC_INFO(1,
+                                        "Transaction successfully committed\n");
+                }
+
+                if (error)
+                        KC_FATAL("%s: %s", what, rd_kafka_error_string(error));
+        }
+#endif
+
+
         /* Wait for all messages to be transmitted */
         conf.run = 1;
         while (conf.run && rd_kafka_outq_len(conf.rk))
@@ -461,11 +527,11 @@ static void handle_partition_eof (rd_kafka_message_t *rkmessage) {
         }
 
         KC_INFO(1, "Reached end of topic %s [%"PRId32"] "
-             "at offset %"PRId64"%s\n",
-             rd_kafka_topic_name(rkmessage->rkt),
-             rkmessage->partition,
-             rkmessage->offset,
-             !conf.run ? ": exiting" : "");
+                "at offset %"PRId64"%s\n",
+                rd_kafka_topic_name(rkmessage->rkt),
+                rkmessage->partition,
+                rkmessage->offset,
+                !conf.run ? ": exiting" : "");
 }
 
 
@@ -486,12 +552,12 @@ static void consume_cb (rd_kafka_message_t *rkmessage, void *opaque) {
 
                 if (rkmessage->rkt)
                         KC_FATAL("Topic %s [%"PRId32"] error: %s",
-                              rd_kafka_topic_name(rkmessage->rkt),
-                              rkmessage->partition,
-                              rd_kafka_message_errstr(rkmessage));
+                                 rd_kafka_topic_name(rkmessage->rkt),
+                                 rkmessage->partition,
+                                 rd_kafka_message_errstr(rkmessage));
                 else
                         KC_FATAL("Consumer error: %s",
-                              rd_kafka_message_errstr(rkmessage));
+                                 rd_kafka_message_errstr(rkmessage));
 
         }
 
@@ -500,11 +566,11 @@ static void consume_cb (rd_kafka_message_t *rkmessage, void *opaque) {
                 if (ts >= conf.stopts) {
                         stop_partition(rkmessage);
                         KC_INFO(1, "Reached stop timestamp for topic %s [%"PRId32"] "
-                             "at offset %"PRId64"%s\n",
-                             rd_kafka_topic_name(rkmessage->rkt),
-                             rkmessage->partition,
-                             rkmessage->offset,
-                             !conf.run ? ": exiting" : "");
+                                "at offset %"PRId64"%s\n",
+                                rd_kafka_topic_name(rkmessage->rkt),
+                                rkmessage->partition,
+                                rkmessage->offset,
+                                !conf.run ? ": exiting" : "");
                         return;                       
                 }
         }
@@ -529,7 +595,7 @@ static void consume_cb (rd_kafka_message_t *rkmessage, void *opaque) {
 static void throttle_cb (rd_kafka_t *rk, const char *broker_name,
                          int32_t broker_id, int throttle_time_ms, void *opaque){
         KC_INFO(1, "Broker %s (%"PRId32") throttled request for %dms\n",
-             broker_name, broker_id, throttle_time_ms);
+                broker_name, broker_id, throttle_time_ms);
 }
 #endif
 
@@ -552,30 +618,35 @@ static void rebalance_cb (rd_kafka_t *rk, rd_kafka_resp_err_t err,
                           void *opaque) {
 
         KC_INFO(1, "Group %s rebalanced (memberid %s): ",
-             conf.group, rd_kafka_memberid(rk));
+                conf.group, rd_kafka_memberid(rk));
 
-	switch (err)
-	{
-	case RD_KAFKA_RESP_ERR__ASSIGN_PARTITIONS:
+        switch (err)
+        {
+        case RD_KAFKA_RESP_ERR__ASSIGN_PARTITIONS:
                 if (conf.verbosity >= 1) {
                         fprintf(stderr, "assigned: ");
                         print_partition_list(1, partitions);
                 }
-		rd_kafka_assign(rk, partitions);
-		break;
+                if (conf.offset != RD_KAFKA_OFFSET_INVALID) {
+                        int i;
+                        for (i = 0 ; i < partitions->cnt ; i++)
+                                partitions->elems[i].offset = conf.offset;
+                }
+                rd_kafka_assign(rk, partitions);
+                break;
 
-	case RD_KAFKA_RESP_ERR__REVOKE_PARTITIONS:
+        case RD_KAFKA_RESP_ERR__REVOKE_PARTITIONS:
                 if (conf.verbosity >= 1) {
                         fprintf(stderr, "revoked: ");
                         print_partition_list(1, partitions);
                 }
-		rd_kafka_assign(rk, NULL);
-		break;
+                rd_kafka_assign(rk, NULL);
+                break;
 
-	default:
+        default:
                 KC_INFO(0, "failed: %s\n", rd_kafka_err2str(err));
-		break;
-	}
+                break;
+        }
 }
 
 /**
@@ -614,7 +685,9 @@ static void kafkaconsumer_run (FILE *fp, char *const *topics, int topic_cnt) {
         /* Subscribe */
         if ((err = rd_kafka_subscribe(conf.rk, topiclist)))
                 KC_FATAL("Failed to subscribe to %d topics: %s\n",
-                      topiclist->cnt, rd_kafka_err2str(err));
+                         topiclist->cnt, rd_kafka_err2str(err));
+
+        KC_INFO(1, "Waiting for group rebalance\n");
 
         rd_kafka_topic_partition_list_destroy(topiclist);
 
@@ -651,7 +724,7 @@ static int64_t *get_offsets(rd_kafka_metadata_topic_t *topic) {
         int64_t *offsets;
         rd_kafka_resp_err_t err;
         rd_kafka_topic_partition_list_t *rktparlistp =
-                                           rd_kafka_topic_partition_list_new(1);
+                rd_kafka_topic_partition_list_new(1);
 
         for (i = 0 ; i < topic->partition_cnt ; i++) {
                 int32_t partition = topic->partitions[i].id;
@@ -662,13 +735,14 @@ static int64_t *get_offsets(rd_kafka_metadata_topic_t *topic) {
                         continue;
 
                 rd_kafka_topic_partition_list_add(rktparlistp,
-                    rd_kafka_topic_name(conf.rkt),
-                    partition)->offset = conf.startts;
+                                                  rd_kafka_topic_name(conf.rkt),
+                                                  partition)->offset = conf.startts;
 
                 if (conf.partition != RD_KAFKA_PARTITION_UA)
                         break;
         }
-        err = rd_kafka_offsets_for_times(conf.rk, rktparlistp, 10*1000);
+        err = rd_kafka_offsets_for_times(conf.rk, rktparlistp,
+                                         conf.metadata_timeout * 1000);
         if (err)
                 KC_FATAL("offsets_for_times failed: %s", rd_kafka_err2str(err));
 
@@ -720,28 +794,29 @@ static void consumer_run (FILE *fp) {
 
 
         /* Query broker for topic + partition information. */
-        if ((err = rd_kafka_metadata(conf.rk, 0, conf.rkt, &metadata, 5000)))
+        if ((err = rd_kafka_metadata(conf.rk, 0, conf.rkt, &metadata,
+                                     conf.metadata_timeout * 1000)))
                 KC_FATAL("Failed to query metadata for topic %s: %s",
-                      rd_kafka_topic_name(conf.rkt), rd_kafka_err2str(err));
+                         rd_kafka_topic_name(conf.rkt), rd_kafka_err2str(err));
 
         /* Error handling */
         if (metadata->topic_cnt == 0)
                 KC_FATAL("No such topic in cluster: %s",
-                      rd_kafka_topic_name(conf.rkt));
+                         rd_kafka_topic_name(conf.rkt));
 
         if ((err = metadata->topics[0].err))
                 KC_FATAL("Topic %s error: %s",
-                      rd_kafka_topic_name(conf.rkt), rd_kafka_err2str(err));
+                         rd_kafka_topic_name(conf.rkt), rd_kafka_err2str(err));
 
         if (metadata->topics[0].partition_cnt == 0)
                 KC_FATAL("Topic %s has no partitions",
-                      rd_kafka_topic_name(conf.rkt));
+                         rd_kafka_topic_name(conf.rkt));
 
         /* If Exit-at-EOF is enabled, set up array to track EOF
          * state for each partition. */
         if (conf.exit_eof || conf.stopts) {
                 part_stop = calloc(sizeof(*part_stop),
-                                  metadata->topics[0].partition_cnt);
+                                   metadata->topics[0].partition_cnt);
 
                 if (conf.partition != RD_KAFKA_PARTITION_UA)
                         part_stop_thres = 1;
@@ -751,7 +826,7 @@ static void consumer_run (FILE *fp) {
 
 #if RD_KAFKA_VERSION >= 0x00090300
         if (conf.startts) {
-            offsets = get_offsets(&metadata->topics[0]);
+                offsets = get_offsets(&metadata->topics[0]);
         }
 #endif
 
@@ -770,7 +845,11 @@ static void consumer_run (FILE *fp) {
 
                 /* Start consumer for this partition */
                 if (rd_kafka_consume_start_queue(conf.rkt, partition,
-                                                 offsets ? offsets[i] : conf.offset,
+                                                 offsets ? offsets[i] :
+                                                 (conf.offset ==
+                                                  RD_KAFKA_OFFSET_INVALID ?
+                                                  RD_KAFKA_OFFSET_BEGINNING :
+                                                  conf.offset),
                                                  rkqu) == -1)
                         KC_FATAL("Failed to start consuming "
                                  "topic %s [%"PRId32"]: %s",
@@ -785,10 +864,10 @@ static void consumer_run (FILE *fp) {
         if (conf.partition != RD_KAFKA_PARTITION_UA &&
             i == metadata->topics[0].partition_cnt)
                 KC_FATAL("Topic %s (with partitions 0..%i): "
-                      "partition %i does not exist",
-                      rd_kafka_topic_name(conf.rkt),
-                      metadata->topics[0].partition_cnt-1,
-                      conf.partition);
+                         "partition %i does not exist",
+                         rd_kafka_topic_name(conf.rkt),
+                         metadata->topics[0].partition_cnt-1,
+                         conf.partition);
 
 
         /* Read messages from Kafka, write to 'fp'. */
@@ -809,9 +888,9 @@ static void consumer_run (FILE *fp) {
                     conf.partition != partition)
                         continue;
 
-		/* Dont stop already stopped partitions */
-		if (!part_stop || !part_stop[partition])
-			rd_kafka_consume_stop(conf.rkt, partition);
+                /* Dont stop already stopped partitions */
+                if (!part_stop || !part_stop[partition])
+                        rd_kafka_consume_stop(conf.rkt, partition);
 
                 rd_kafka_consume_stop(conf.rkt, partition);
         }
@@ -922,7 +1001,7 @@ static void metadata_list (void) {
 
         /* Fetch metadata */
         err = rd_kafka_metadata(conf.rk, conf.rkt ? 0 : 1, conf.rkt,
-                                &metadata, 5000);
+                                &metadata, conf.metadata_timeout * 1000);
         if (err != RD_KAFKA_RESP_ERR_NO_ERROR)
                 KC_FATAL("Failed to acquire metadata: %s", rd_kafka_err2str(err));
 
@@ -988,6 +1067,9 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
 #endif
 #if ENABLE_AVRO
                 "Avro, "
+#endif
+#if ENABLE_TXNS
+                "Transactions, "
 #endif
                 ,
                 rd_kafka_version_str(), features
@@ -1013,6 +1095,13 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
                 "  -E                 Do not exit on non fatal error\n"
                 "  -K <delim>         Key delimiter (same format as -D)\n"
                 "  -c <cnt>           Limit message count\n"
+                "  -m <seconds>       Metadata (et.al.) request timeout.\n"
+                "                     This limits how long kafkacat will block\n"
+                "                     while waiting for initial metadata to be\n"
+                "                     retrieved from the Kafka cluster.\n"
+                "                     It also sets the timeout for the producer's\n"
+                "                     transaction commits, init, aborts, etc.\n"
+                "                     Default: 5 seconds.\n"
                 "  -F <config-file>   Read configuration properties from file,\n"
                 "                     file format is \"property=value\".\n"
                 "                     The KAFKACAT_CONFIG=path environment can "
@@ -1057,6 +1146,12 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
                 "                     With -l, only one file permitted.\n"
                 "                     Otherwise, the entire file contents will\n"
                 "                     be sent as one single message.\n"
+                "  -X transactional.id=.. Enable transactions and send all\n"
+                "                     messages in a single transaction which\n"
+                "                     is committed when stdin is closed or the\n"
+                "                     input file(s) are fully read.\n"
+                "                     If kafkacat is terminated through Ctrl-C\n"
+                "                     (et.al) the transaction will be aborted.\n"
                 "\n"
                 "Consumer options:\n"
                 "  -o <offset>        Offset to start consuming from:\n"
@@ -1110,7 +1205,7 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
                 "  -O                 Print message offset using -K delimiter\n"
                 "  -c <cnt>           Exit after consuming this number "
                 "of messages\n"
-                "  -Z                 Print NULL values and keys as \"%s\""
+                "  -Z                 Print NULL values and keys as \"%s\" "
                 "instead of empty.\n"
                 "                     For JSON (-J) the nullstr is always null.\n"
                 "  -u                 Unbuffered output\n"
@@ -1148,16 +1243,19 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
                 " Example:\n"
                 "  -f 'Topic %%t [%%p] at offset %%o: key %%k: %%s\\n'\n"
                 "\n"
+#if ENABLE_JSON
                 "JSON message envelope (on one line) when consuming with -J:\n"
                 " { \"topic\": str, \"partition\": int, \"offset\": int,\n"
                 "   \"tstype\": \"create|logappend|unknown\", \"ts\": int, "
                 "// timestamp in milliseconds since epoch\n"
+                "   \"broker\": int,\n"
                 "   \"headers\": { \"<name>\": str, .. }, // optional\n"
                 "   \"key\": str|json, \"payload\": str|json,\n"
                 "   \"key_error\": str, \"payload_error\": str } //optional\n"
                 " (note: key_error and payload_error are only included if "
                 "deserialization failed)\n"
                 "\n"
+#endif
                 "Consumer mode (writes messages to stdout):\n"
                 "  kafkacat -b <broker> -t <topic> -p <partition>\n"
                 " or:\n"
@@ -1190,6 +1288,7 @@ static void RD_NORETURN usage (const char *argv0, int exitcode,
  */
 static void term (int sig) {
         conf.run = 0;
+        conf.term_sig = sig;
 }
 
 
@@ -1208,20 +1307,20 @@ static void error_cb (rd_kafka_t *rk, int err,
                 rd_kafka_resp_err_t fatal_err;
 
                 fatal_err = rd_kafka_fatal_error(rk, fatal_errstr,
-                                           sizeof(fatal_errstr));
+                                                 sizeof(fatal_errstr));
                 KC_INFO(0, "FATAL CLIENT ERROR: %s: %s: terminating\n",
                         rd_kafka_err2str(fatal_err), fatal_errstr);
                 conf.run = 0;
 
         } else
 #endif
-        if (err == RD_KAFKA_RESP_ERR__ALL_BROKERS_DOWN) {
-                KC_ERROR("%s: %s", rd_kafka_err2str(err),
-                         reason ? reason : "");
-        } else {
-                KC_INFO(1, "ERROR: %s: %s\n", rd_kafka_err2str(err),
-                        reason ? reason : "");
-        }
+                if (err == RD_KAFKA_RESP_ERR__ALL_BROKERS_DOWN) {
+                        KC_ERROR("%s: %s", rd_kafka_err2str(err),
+                                 reason ? reason : "");
+                } else {
+                        KC_INFO(1, "ERROR: %s: %s\n", rd_kafka_err2str(err),
+                                reason ? reason : "");
+                }
 }
 
 
@@ -1322,8 +1421,10 @@ static int try_conf_set (const char *name, const char *val,
                 if (!conf.srconf)
                         conf.srconf = serdes_conf_new(NULL, 0, NULL);
 
-                if (!strcmp(name, "schema.registry.url"))
+                if (!strcmp(name, "schema.registry.url")) {
+                        conf.flags |= CONF_F_SR_URL_SEEN;
                         srlen = 0;
+                }
 
                 serr = serdes_conf_set(conf.srconf, name+srlen, val,
                                        errstr, errstr_size);
@@ -1571,7 +1672,7 @@ static void argparse (int argc, char **argv,
 
         while ((opt = getopt(argc, argv,
                              ":PCG:LQt:p:b:z:o:eED:K:k:H:Od:qvF:X:c:Tuf:ZlVh"
-                             "s:r:J")) != -1) {
+                             "s:r:Jm:")) != -1) {
                 switch (opt) {
                 case 'P':
                 case 'C':
@@ -1683,6 +1784,7 @@ static void argparse (int argc, char **argv,
                         if (!*optarg)
                                 KC_FATAL("-s url must not be empty");
                         conf.schema_registry_url = optarg;
+                        conf.flags |= CONF_F_SR_URL_SEEN;
 #else
                         KC_FATAL("This build of kafkacat lacks "
                                  "Avro/Schema-Registry support");
@@ -1711,6 +1813,9 @@ static void argparse (int argc, char **argv,
                 case 'c':
                         conf.msg_cnt = strtoll(optarg, NULL, 10);
                         break;
+                case 'm':
+                        conf.metadata_timeout = strtoll(optarg, NULL, 10);
+                        break;
                 case 'Z':
                         conf.flags |= CONF_F_NULL;
                         conf.null_str_len = strlen(conf.null_str);
@@ -1817,7 +1922,7 @@ static void argparse (int argc, char **argv,
                 else
                         conf.mode = 'P';
                 KC_INFO(1, "Auto-selecting %s mode (use -P or -C to override)\n",
-                     conf.mode == 'C' ? "Consumer":"Producer");
+                        conf.mode == 'C' ? "Consumer":"Producer");
         }
 
 
@@ -1882,7 +1987,7 @@ static void argparse (int argc, char **argv,
          * Verify and initialize Avro/SR
          */
 #if ENABLE_AVRO
-        if (!!conf.schema_registry_url !=
+        if (!!(conf.flags & CONF_F_SR_URL_SEEN) !=
             !!(conf.flags & (CONF_F_FMT_AVRO_VALUE|CONF_F_FMT_AVRO_KEY)))
                 KC_FATAL("-r requires -s avro and vice-versa");
 
@@ -1939,8 +2044,8 @@ static void argparse (int argc, char **argv,
 
         } else if (conf.mode == 'P') {
                 conf.delim = parse_delim(delim);
-		if (conf.flags & CONF_F_KEY_DELIM)
-			conf.key_delim = parse_delim(key_delim);
+                if (conf.flags & CONF_F_KEY_DELIM)
+                        conf.key_delim = parse_delim(key_delim);
         }
 
         /* Automatically enable API version requests if needed and
@@ -1958,10 +2063,10 @@ static void argparse (int argc, char **argv,
 
 int main (int argc, char **argv) {
 #ifdef SIGIO
-	char tmp[16];
+        char tmp[16];
 #endif
         FILE *in = stdin;
-	struct timeval tv;
+        struct timeval tv;
         rd_kafka_topic_partition_list_t *rktparlist = NULL;
 
         /* Certain Docker images don't have kafkacat as the entry point,
@@ -1981,9 +2086,9 @@ int main (int argc, char **argv) {
         signal(SIGPIPE, term);
 #endif
 
-	/* Seed rng for random partitioner, jitter, etc. */
-	rd_gettimeofday(&tv, NULL);
-	srand(tv.tv_usec);
+        /* Seed rng for random partitioner, jitter, etc. */
+        rd_gettimeofday(&tv, NULL);
+        srand(tv.tv_usec);
 
         /* Create config containers */
         conf.rk_conf  = rd_kafka_conf_new();
@@ -2029,6 +2134,9 @@ int main (int argc, char **argv) {
 
 #if ENABLE_KAFKACONSUMER
         case 'G':
+                if (conf.stopts || conf.startts)
+                        KC_FATAL("-o ..@ timestamps can't be used "
+                                 "with -G mode\n");
                 kafkaconsumer_run(stdout, &argv[optind], argc-optind);
                 break;
 #endif
