@@ -94,13 +94,14 @@ struct conf {
 #define CONF_F_OFFSET     0x4 /* Print offsets */
 #define CONF_F_TEE        0x8 /* Tee output when producing */
 #define CONF_F_NULL       0x10 /* -Z: Send empty messages as NULL */
-#define CONF_F_LINE	  0x20 /* Read files in line mode when producing */
+#define CONF_F_LINE       0x20 /* Read files in line mode when producing */
 #define CONF_F_APIVERREQ  0x40 /* Enable api.version.request=true */
 #define CONF_F_APIVERREQ_USER 0x80 /* User set api.version.request */
 #define CONF_F_NO_CONF_SEARCH 0x100 /* Disable default config file search */
 #define CONF_F_BROKERS_SEEN   0x200 /* Brokers have been configured */
 #define CONF_F_FMT_AVRO_KEY   0x400 /* Convert key from Avro to JSON */
 #define CONF_F_FMT_AVRO_VALUE 0x800 /* Convert value from Avro to JSON  */
+#define CONF_F_SR_URL_SEEN    0x1000 /* schema.registry.url/-r seen */
         int     delim;
         int     key_delim;
 
@@ -130,8 +131,10 @@ struct conf {
 #endif
         int     exit_eof;
         int64_t msg_cnt;
+        int     metadata_timeout;
         char   *null_str;
         int     null_str_len;
+        int     txn;
 
         rd_kafka_conf_t       *rk_conf;
         rd_kafka_topic_conf_t *rkt_conf;
@@ -141,6 +144,8 @@ struct conf {
 
         char   *debug;
 
+        int term_sig;  /**< Termination signal */
+
 #if ENABLE_AVRO
         serdes_conf_t *srconf;
         char   *schema_registry_url;
@@ -151,19 +156,19 @@ extern struct conf conf;
 
 
 void RD_NORETURN fatal0 (const char *func, int line,
-                                       const char *fmt, ...);
+                         const char *fmt, ...);
 
 void error0 (int erroronexit, const char *func, int line,
-                                       const char *fmt, ...);
+             const char *fmt, ...);
 
 #define KC_FATAL(.../*fmt*/)  fatal0(__FUNCTION__, __LINE__, __VA_ARGS__)
 
 #define KC_ERROR(.../*fmt*/)  error0(conf.exitonerror, __FUNCTION__, __LINE__, __VA_ARGS__)
 
 /* Info printout */
-#define KC_INFO(VERBLVL,.../*fmt*/) do {                    \
-                if (conf.verbosity >= (VERBLVL))     \
-                        fprintf(stderr, "%% " __VA_ARGS__);  \
+#define KC_INFO(VERBLVL,.../*fmt*/) do {                        \
+                if (conf.verbosity >= (VERBLVL))                \
+                        fprintf(stderr, "%% " __VA_ARGS__);     \
         } while (0)
 
 
